"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_spawner_1 = require("../../util/spawner/child-spawner");
const winston = require('winston');
const util = require('util');
const path = require('path');
const fs = require('fs');
const process = require("process");
const alwayscli_1 = require("@alwaysai/alwayscli");
const js_spawner_1 = require("../../util/spawner/js-spawner");
const child_process_1 = require("child_process");
const signalExit = require("signal-exit");
// Runs command with shell=true, neccesary for commands that require spaces
async function runForegroundSync(cmd) {
    winston.debug(cmd);
    child_process_1.spawnSync(cmd, [], {
        stdio: 'inherit',
        shell: true,
    });
}
async function reAttachHandleDown(composeCmd, composeYamlPath) {
    const args = ['-f', `${composeYamlPath}`, 'logs', '-f'];
    winston.debug(`${composeCmd} ${args.join(' ')}`);
    const logFile = fs.createWriteStream('annotate.log', { flags: 'w' });
    const childProcess = child_process_1.spawn(composeCmd, args);
    function stdoutChunkHandler(chunk) {
        logFile.write(`${util.format(chunk.toString())} \n`);
    }
    childProcess.stdout.on('data', stdoutChunkHandler);
    signalExit(() => {
        process.on('SIGINT', () => {
            const cvatDownCommand = `${composeCmd} -f ${composeYamlPath} down`;
            winston.debug(cvatDownCommand);
            child_process_1.spawnSync(cvatDownCommand, [], {
                stdio: 'inherit',
                shell: true,
            });
        });
    });
    return await new Promise((resolve, reject) => {
        childProcess.on('error', err => {
            reject(err);
        });
        childProcess.on('exit', code => {
            if (code) {
                resolve(code);
            }
            else {
                resolve();
            }
        });
    });
}
const datasetAnnotateFunc = () => alwayscli_1.CliLeaf({
    name: 'annotate',
    description: 'Run annotation tool',
    async action() {
        const aaiLibPath = path.dirname(require.main.filename);
        const spawner = js_spawner_1.JsSpawner();
        let composeCommand = 'docker-compose';
        // Install docker-compose if not previously installed
        {
            try {
                await spawner.run({ exe: composeCommand, args: ['--version'] });
                winston.debug('Global docker-compose found. Skipping installation.');
            }
            catch (ex) {
                // Check local docker-compose
                composeCommand = path.join(aaiLibPath, 'docker-compose');
                try {
                    await spawner.run({ exe: composeCommand, args: ['--version'] });
                    winston.debug('Local docker-compose found. Skipping installation.');
                }
                catch (ex) {
                    winston.info(`docker-compose not installed, downloading to ${composeCommand} for use with cli`);
                    const os = await child_spawner_1.ChildSpawner({ path: '.' }).run({
                        exe: 'uname',
                        args: ['-s'],
                    });
                    const machine = await child_spawner_1.ChildSpawner({ path: '.' }).run({
                        exe: 'uname',
                        args: ['-m'],
                    });
                    await spawner.run({
                        exe: 'curl',
                        args: [
                            '-L',
                            `https://github.com/docker/compose/releases/download/1.25.4/docker-compose-${os}-${machine}`,
                            '-o',
                            composeCommand,
                        ],
                    });
                    await spawner.run({ exe: 'chmod', args: ['+x', composeCommand] });
                    try {
                        await spawner.run({ exe: composeCommand, args: ['--version'] });
                    }
                    catch (ex) {
                        winston.error('Could not install docker-compose which is needed for this command. Please manually install.');
                    }
                }
            }
        }
        // Run docker-compose up to bring up CVAT
        const composeYamlPath = `${path.join(aaiLibPath, 'subcommands', 'dataset', 'cvat', 'docker-compose.yml')}`;
        await runForegroundSync(`${composeCommand} -f ${composeYamlPath} pull`);
        await runForegroundSync(`${composeCommand} -f ${composeYamlPath} --log-level CRITICAL up -d`);
        winston.info('Running CVAT http://localhost:8080');
        // If first run Add superuser
        const superUserCreatedPath = path.join(aaiLibPath, 'super-user-created');
        if (!fs.existsSync(superUserCreatedPath)) {
            await runForegroundSync("docker exec -it cvat bash -ic 'python3 ~/manage.py createsuperuser'");
            await runForegroundSync(`touch ${superUserCreatedPath}`);
        }
        // Re-attach to running cvat and call down if we ctrl-c out
        await reAttachHandleDown(composeCommand, composeYamlPath);
    },
});
exports.datasetAnnotate = datasetAnnotateFunc();
exports.datasetAnnotateAlias = datasetAnnotateFunc();
exports.datasetAnnotateAlias.description = 'Alias for dataset annotate';
//# sourceMappingURL=annotate.js.map