"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const model_id_1 = require("./model-id");
const model_json_file_1 = require("./model-json-file");
const random_string_1 = require("./random-string");
const model_package_cache_1 = require("./model-package-cache");
const download_model_package_to_cache_1 = require("./download-model-package-to-cache");
const constants_1 = require("../constants");
async function appInstallModel(target, id, version) {
    const { publisher, name } = model_id_1.ModelId.parse(id);
    const destinationDir = path_1.posix.join(constants_1.APP_MODELS_DIRECTORY_NAME, publisher, name);
    let installedVersion = undefined;
    try {
        const parsed = await readModelJson(destinationDir);
        installedVersion = parsed.version;
    }
    catch (_) {
        // TODO finer-grained error handling
    }
    if (installedVersion !== version) {
        const tmpId = random_string_1.RandomString();
        const tmpDir = `${destinationDir}.${tmpId}.tmp`;
        await target.mkdirp(tmpDir);
        try {
            if (!model_package_cache_1.modelPackageCache.has(id, version)) {
                await download_model_package_to_cache_1.downloadModelPackageToCache(id, version);
            }
            const modelPackageStream = await model_package_cache_1.modelPackageCache.read(id, version);
            await target.untar(modelPackageStream, tmpDir);
            const fileNames = await target.readdir(tmpDir);
            // Sanity check
            if (fileNames.length !== 1 || !fileNames[0]) {
                throw new Error('Expected package to contain single directory');
            }
            // The model json file in the package does not have the version number,
            // so we write it into the json file during install
            await updateModelJson(path_1.posix.join(tmpDir, fileNames[0]), modelJson => (Object.assign(Object.assign({}, modelJson), { version })));
            await target.rimraf(destinationDir);
            await target.mkdirp(path_1.dirname(destinationDir));
            await target.rename(target.resolvePath(tmpDir, fileNames[0]), destinationDir);
        }
        catch (exception) {
            try {
                await target.rimraf(tmpDir);
            }
            finally {
                throw exception;
            }
        }
    }
    async function readModelJson(dir) {
        const filePath = target.resolvePath(dir, model_json_file_1.MODEL_JSON_FILE_NAME);
        const output = await target.readFile(filePath);
        const parsed = JSON.parse(output);
        return parsed;
    }
    async function updateModelJson(dir, updater) {
        const parsed = await readModelJson(dir);
        const updated = updater(parsed);
        const filePath = target.resolvePath(dir, model_json_file_1.MODEL_JSON_FILE_NAME);
        const serialized = JSON.stringify(updated, null, 2);
        await target.writeFile(filePath, serialized);
    }
}
exports.appInstallModel = appInstallModel;
//# sourceMappingURL=app-install-model.js.map