"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_1 = require("fs");
const node_fetch_1 = require("node-fetch");
const cloud_api_1 = require("@alwaysai/cloud-api");
const coded_error_1 = require("@carnesen/coded-error");
const tar = require("tar");
const rpc_client_1 = require("./rpc-client");
const authentication_client_1 = require("./authentication-client");
const model_json_file_1 = require("./model-json-file");
const util_1 = require("util");
const tempy = require("tempy");
const system_id_1 = require("./system-id");
exports.modelPackageCloudClient = {
    async download(id, version) {
        const { uuid } = await rpc_client_1.CliRpcClient().getModelVersion({ id, version });
        const modelPackageUrl = ModelPackageUrl(uuid);
        const authorizationHeader = await authentication_client_1.CliAuthenticationClient().getAuthorizationHeader();
        const response = await node_fetch_1.default(modelPackageUrl, {
            method: 'GET',
            headers: Object.assign(Object.assign({}, authorizationHeader), { 'Content-Length': '0' }),
        });
        await throwIfNotOk(response);
        return response.body;
    },
    async publish(dir = process.cwd()) {
        const modelJson = model_json_file_1.ModelJsonFile(dir).read();
        const rpcClient = rpc_client_1.CliRpcClient();
        const { uuid } = await rpcClient.createModelVersion(modelJson);
        const authorizationHeader = await authentication_client_1.CliAuthenticationClient().getAuthorizationHeader();
        const modelPackagePath = tempy.file();
        // Note: The TypeScript types on tar.create are not 100% accurate
        await tar.create({ cwd: path_1.dirname(dir), gzip: true, file: modelPackagePath }, [
            path_1.basename(dir),
        ]);
        const stats = await util_1.promisify(fs_1.stat)(modelPackagePath);
        const modelPackageStream = fs_1.createReadStream(modelPackagePath);
        const response = await node_fetch_1.default(ModelPackageUrl(uuid), {
            method: 'PUT',
            headers: Object.assign({ 'Content-Type': 'application/gzip', 'Content-Length': stats.size.toString() }, authorizationHeader),
            body: modelPackageStream,
        });
        await throwIfNotOk(response);
        await rpcClient.finalizeModelVersion(uuid);
        return uuid;
    },
};
function ModelPackageUrl(uuid) {
    const cloudApiUrl = cloud_api_1.CloudApiUrl(system_id_1.getSystemId());
    return `${cloudApiUrl}${cloud_api_1.CLOUD_API_MODEL_VERSION_PACKAGES_PATH}/${uuid}`;
}
async function throwIfNotOk(response) {
    if (!response.ok) {
        const parsed = await response.json();
        const { message = `Server responded ${response.status} ("${response.statusText}")`, code, data, } = parsed;
        throw new coded_error_1.CodedError(message, code, data);
    }
}
//# sourceMappingURL=model-package-cloud-client.js.map