"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const chalk_1 = require("chalk");
const t = require("io-ts");
const config_nodejs_1 = require("@alwaysai/config-nodejs");
const alwayscli_1 = require("@alwaysai/alwayscli");
const docker_spawner_1 = require("./spawner/docker-spawner");
const target_protocol_1 = require("./target-protocol");
const ssh_docker_spawner_1 = require("./spawner/ssh-docker-spawner");
const ssh_spawner_1 = require("./spawner/ssh-spawner");
const js_spawner_1 = require("./spawner/js-spawner");
const constants_1 = require("../constants");
const sshDockerTargetJsonCodec = t.type({
    targetProtocol: t.literal(target_protocol_1.TargetProtocol['ssh+docker:']),
    targetHostname: t.string,
    targetPath: t.string,
    dockerImageId: t.string,
    targetHardware: t.string,
}, 'SshDockerTarget');
const dockerTargetJsonCodec = t.type({
    targetProtocol: t.literal(target_protocol_1.TargetProtocol['docker:']),
    dockerImageId: t.string,
    targetHardware: t.string,
});
const targetJsonCodec = t.taggedUnion('protocol', [
    dockerTargetJsonCodec,
    sshDockerTargetJsonCodec,
]);
const DID_YOU_RUN_APP_CONFIGURE = 'Did you run "alwaysai app configure"?';
const ENOENT = {
    message: `${constants_1.TARGET_JSON_FILE_NAME} not found. ${DID_YOU_RUN_APP_CONFIGURE}`,
    code: alwayscli_1.CLI_TERSE_ERROR,
};
function TargetJsonFile(cwd = process.cwd()) {
    const filePath = path_1.join(cwd, constants_1.TARGET_JSON_FILE_NAME);
    const configFile = config_nodejs_1.ConfigFile({ path: filePath, codec: targetJsonCodec, ENOENT });
    return Object.assign(Object.assign({}, configFile), { readContainerSpawner,
        readHostSpawner,
        describe });
    function describe() {
        const config = configFile.readIfExists();
        if (!config) {
            return `Target configuration file "${constants_1.TARGET_JSON_FILE_NAME}" not found`;
        }
        const docker = chalk_1.default.bold('docker');
        switch (config.targetProtocol) {
            case 'docker:': {
                return `Target: ${docker} container on this host`;
            }
            case 'ssh+docker:': {
                const hostname = chalk_1.default.bold(config.targetHostname);
                const path = chalk_1.default.bold(config.targetPath);
                return `Target: ${docker} container on ${hostname}, path ${path}`;
            }
            default:
                throw new Error('Unexpected protocol');
        }
    }
    function readContainerSpawner() {
        const targetJson = configFile.read();
        switch (targetJson.targetProtocol) {
            case 'ssh+docker:': {
                const { targetHostname, targetPath, dockerImageId } = targetJson;
                return ssh_docker_spawner_1.SshDockerSpawner({
                    dockerImageId,
                    targetPath,
                    targetHostname,
                });
            }
            case 'docker:': {
                const { dockerImageId } = targetJson;
                return docker_spawner_1.DockerSpawner({ dockerImageId });
            }
            default:
                throw new alwayscli_1.CliTerseError('Unsupported protocol');
        }
    }
    function readHostSpawner() {
        const targetJson = configFile.read();
        switch (targetJson.targetProtocol) {
            case 'ssh+docker:': {
                const { targetPath, targetHostname } = targetJson;
                return ssh_spawner_1.SshSpawner({
                    targetHostname,
                    targetPath,
                });
            }
            case 'docker:': {
                return js_spawner_1.JsSpawner();
            }
            default:
                throw new alwayscli_1.CliTerseError('Unsupported protocol');
        }
    }
}
exports.TargetJsonFile = TargetJsonFile;
//# sourceMappingURL=target-json-file.js.map