import { ChildSpawner } from '../../util/spawner/child-spawner';

const winston = require('winston');
const util = require('util');
const path = require('path');
const fs = require('fs');
import process = require('process');

import { CliLeaf } from '@alwaysai/alwayscli';
import { JsSpawner } from '../../util/spawner/js-spawner';
import { spawn, spawnSync } from 'child_process';
import signalExit = require('signal-exit');

// Runs command with shell=true, neccesary for commands that require spaces
async function runForegroundSync(cmd: string) {
  winston.debug(cmd);
  spawnSync(cmd, [], {
    stdio: 'inherit',
    shell: true,
  });
}

async function reAttachHandleDown(composeCmd: string, composeYamlPath: string) {
  const args = ['-f', `${composeYamlPath}`, 'logs', '-f'];
  winston.debug(`${composeCmd} ${args.join(' ')}`);
  const logFile = fs.createWriteStream('annotate.log', { flags: 'w' });
  const childProcess = spawn(composeCmd, args);
  function stdoutChunkHandler(chunk: Buffer) {
    logFile.write(`${util.format(chunk.toString())} \n`);
  }
  childProcess.stdout.on('data', stdoutChunkHandler);
  signalExit(() => {
    process.on('SIGINT', () => {
      const cvatDownCommand = `${composeCmd} -f ${composeYamlPath} down`;
      winston.debug(cvatDownCommand);
      spawnSync(cvatDownCommand, [], {
        stdio: 'inherit',
        shell: true,
      });
    });
  });
  return await new Promise<number | undefined>((resolve, reject) => {
    childProcess.on('error', err => {
      reject(err);
    });

    childProcess.on('exit', code => {
      if (code) {
        resolve(code);
      } else {
        resolve();
      }
    });
  });
}

const datasetAnnotateFunc = () =>
  CliLeaf({
    name: 'annotate',
    description: 'Run annotation tool',
    async action() {
      const aaiLibPath = path.dirname(require.main!.filename);
      const spawner = JsSpawner();
      let composeCommand = 'docker-compose';
      // Install docker-compose if not previously installed
      {
        try {
          await spawner.run({ exe: composeCommand, args: ['--version'] });
          winston.debug('Global docker-compose found. Skipping installation.');
        } catch (ex) {
          // Check local docker-compose
          composeCommand = path.join(aaiLibPath, 'docker-compose');
          try {
            await spawner.run({ exe: composeCommand, args: ['--version'] });
            winston.debug('Local docker-compose found. Skipping installation.');
          } catch (ex) {
            winston.info(
              `docker-compose not installed, downloading to ${composeCommand} for use with cli`,
            );
            const os = await ChildSpawner({ path: '.' }).run({
              exe: 'uname',
              args: ['-s'],
            });
            const machine = await ChildSpawner({ path: '.' }).run({
              exe: 'uname',
              args: ['-m'],
            });
            await spawner.run({
              exe: 'curl',
              args: [
                '-L',
                `https://github.com/docker/compose/releases/download/1.25.4/docker-compose-${os}-${machine}`,
                '-o',
                composeCommand,
              ],
            });
            await spawner.run({ exe: 'chmod', args: ['+x', composeCommand] });
            try {
              await spawner.run({ exe: composeCommand, args: ['--version'] });
            } catch (ex) {
              winston.error(
                'Could not install docker-compose which is needed for this command. Please manually install.',
              );
            }
          }
        }
      }
      // Run docker-compose up to bring up CVAT
      const composeYamlPath = `${path.join(
        aaiLibPath,
        'subcommands',
        'dataset',
        'cvat',
        'docker-compose.yml',
      )}`;
      await runForegroundSync(`${composeCommand} -f ${composeYamlPath} pull`);
      await runForegroundSync(
        `${composeCommand} -f ${composeYamlPath} --log-level CRITICAL up -d`,
      );
      winston.info('Running CVAT http://localhost:8080');
      // If first run Add superuser
      const superUserCreatedPath = path.join(aaiLibPath, 'super-user-created');
      if (!fs.existsSync(superUserCreatedPath)) {
        await runForegroundSync(
          "docker exec -it cvat bash -ic 'python3 ~/manage.py createsuperuser'",
        );
        await runForegroundSync(`touch ${superUserCreatedPath}`);
      }
      // Re-attach to running cvat and call down if we ctrl-c out
      await reAttachHandleDown(composeCommand, composeYamlPath);
    },
  });

export const datasetAnnotate = datasetAnnotateFunc();
export const datasetAnnotateAlias = datasetAnnotateFunc();
datasetAnnotateAlias.description = 'Alias for dataset annotate';
