const fs = require('fs');
const path = require('path');
const os = require('os');
import { spawnSync } from 'child_process';

const AdmZip = require('adm-zip');
import { CliLeaf, CliNumberInput, CliStringArrayInput } from '@alwaysai/alwayscli';
const winston = require('winston');
const parseStringSync = require('xml2js-parser').parseStringSync;

// Runs command with shell=true, necessary for commands that require spaces
async function runForegroundSync(cmd: string) {
  winston.debug(cmd);
  spawnSync(cmd, [], {
    stdio: 'inherit',
    shell: true,
  });
}

function createTrainJson(steps: number, classes: number, labels: string[]) {
  const output = {
    numSteps: steps,
    numClasses: classes,
    labels,
  };

  fs.writeFileSync('training-temp/alwaysai.train.json', JSON.stringify(output, null, 4));
}

function validateLabels(annotationsDir: string, labels: string[]) {
  const annotationFilesNames = fs.readdirSync(annotationsDir);

  // Should create annotationSet before this for loop and check labels after
  const annotationLabels = new Set();
  annotationFilesNames.forEach((annotationFileName: string) => {
    // Increment FileName Field in Annotation File
    const annotationJson_ = parseStringSync(
      fs.readFileSync(path.join(annotationsDir, annotationFileName)),
    );
    const objectList = annotationJson_.annotation.object;
    objectList.forEach((object: any) => {
      annotationLabels.add(object.name[0]);
    });
  });
  winston.info(
    `Available labels for dataset:\n  ${Array.from(annotationLabels).join('\n  ')}`,
  );
  labels.forEach(label => {
    if (!annotationLabels.has(label)) {
      throw `Label "${label}" not found in any annotation files Are you sure you typed them in correctly? Annotation file labels: ${Array.from(
        annotationLabels,
      )}`;
    }
  });
}

const datasetTrainFunc = () =>
  CliLeaf({
    name: 'train',
    description: 'Produce object detection model for dataset',
    positionalInput: CliStringArrayInput({
      placeholder: '</path/to/dataset1.zip> </path/to/dataset2.zip> ...',
      required: true,
    }),
    namedInputs: {
      labels: CliStringArrayInput({
        description: 'Dataset labels to train on',
        required: true,
      }),
      numSteps: CliNumberInput({
        description: 'Train for x number of steps',
        required: true,
      }),
    },
    async action([...args], opts) {
      winston.info(`Datasets\n  ${args.join('\n  ')}`);
      winston.info(`Labels\n  ${opts['labels'].join('\n  ')}`);
      if (args.length === 1) {
        const zip = new AdmZip(args[0]);
        zip.extractAllTo('training-temp', true);
      } else {
        fs.mkdirSync('training-temp');
        await runForegroundSync(`aai dataset merge ${args.join(' ')}`);
        const zip = new AdmZip('merged-dataset.zip');
        zip.extractAllTo('training-temp', true);
      }
      const labels = opts['labels'];
      validateLabels('training-temp/Annotations', labels);
      createTrainJson(opts['numSteps'], labels.length, labels);
      const cpuCount = os.cpus().length;
      await runForegroundSync(
        `docker run -it --cpus ${cpuCount -
          1} -v $\{PWD}/training-temp:/alwaysai alwaysai/train`,
      );
    },
  });

export const datasetTrain = datasetTrainFunc();
export const datasetTrainAlias = datasetTrainFunc();
datasetTrainAlias.description = 'Alias for dataset train';
